from rest_framework.request import Request

from .constants import AUTH_REQUEST_USER_FIELD_NAME, AUTH_REQUEST_PASSWORD_FIELD_NAME
from .results import AuthResult


class MsbAuthenticator:
	_config: object = None
	__auth_request: Request = None

	@property
	def auth_request(self) -> Request:
		return self.__auth_request

	@property
	def auth_user(self):
		return self.auth_request.data.get(AUTH_REQUEST_USER_FIELD_NAME, None)

	@property
	def auth_user_password(self):
		return self.auth_request.data.get(AUTH_REQUEST_PASSWORD_FIELD_NAME, None)

	def __init__(self, user: str, pwd: str, request: Request = None):
		self.__auth_request = request

	def authenticate(self, encoded_pwd: str, **kwargs) -> AuthResult:
		raise NotImplementedError



from django.conf import settings

from msb.dataclasses.notification import (NotificationData, EmailNotificationData, YammerNotificationData)
from .exceptions import NotificationServiceExceptions
from .queue_service import (MsbQueue, MsbService, NameConst)


class NotificationService(MsbService):

	def __init__(self, **kwargs):
		super().__init__()
		self.queue = MsbQueue(
			queue=kwargs.get('queue', settings.MSB_NOTIFICATION_QUEUE_NAME),
			service=kwargs.get('service', settings.MSB_SERVICE_NAME)
		)

	def send_notification_task(self, data: NotificationData = None, task_name: str = NameConst.ASYNC_TASK_NAME):
		try:
			if not isinstance(data, NotificationData):
				raise NotificationServiceExceptions.InvalidNotificationData

			self.queue.publish(task_name=task_name, data=data.__dict__)
		except Exception as e:
			self.raise_exceptions(e, NotificationServiceExceptions.NotificationSendFailed())

	def send_email_notification(self, data: EmailNotificationData = None, task: str = None):
		return self.send_notification_task(
			data, (task or getattr(settings, 'MSB_EMAIL_NOTIFICATION_TASK_NAME', NameConst.EMAIL_NOTIFICATION_TASK_NAME))
		)

	def send_bulk_email_notification(self, data: list = None, task: str = None):
		raise NotImplementedError

	def send_yammer_notification(self, data: YammerNotificationData = None, task: str = None):
		return self.send_notification_task(
			data, (task or getattr(settings, 'MSB_YAMMER_NOTIFICATION_TASK_NAME', NameConst.YAMMER_NOTIFICATION_TASK_NAME))
		)


__all__ = ['NotificationService', 'NotificationData', 'EmailNotificationData', 'YammerNotificationData']



"""
HTTP HEADER NAMES
"""

HEADER_NAME_CONTENT_TYPE = "Content-Type"
HEADER_NAME_AUTHORIZATION = "Authorization"
AUTHORIZATION_NAME_BEARER_TOKEN = "Bearer"

"""
HTTP CONTENT TYPE NAMES
"""
CONTENT_TYPE_APPLICATION_JSON = "application/json"
AUTHORIZATION_BEARER_TOKEN_STRING = "Bearer {token}"


class HttpContentType:
	APPLICATION_JSON = "application/json"


class HttpHeaders:
	CONTENT_TYPE = "Content-Type"
	AUTHORIZATION = "Authorization"
	BEARER_TOKEN_VALUE = "Bearer {token}"


class RequestMethod:
	GET = "get"
	POST = "post"
	PUT = "put"
	DELETE = "delete"


DEFAULT_RESPONSE_HEADERS = {
	HttpHeaders.CONTENT_TYPE: HttpContentType.APPLICATION_JSON
}



from msb.apis import (ApiView, api_details, ApiViewset, )
from msb.auth import (Permissions, api_permissions, TokenUser, SessionData)
from msb.cipher import (Cipher)
from msb.dataclasses import (SearchParameter, SearchParameterRequest, Singleton)
from msb.db.models import (MsbModel, MsbModelMetaFields, MsbModelManager, model_fields)
from msb.exceptions import (ApiException, AppException, CrudApiException)
from msb.http import (RequestHeaders, RestResponse, RequestInfo, ApiResponse)
from msb.services import (ApiService, MsbService, ApiServiceExceptions, CrudApiException)
from msb.validation import (api_inputs, DefaultRules, ValidationSchema, InputField, Validate, validation_schema_wrapper)

__all__ = [

	# core msb_apis/service imports
	"ApiView", "ApiViewset", "api_details", "ApiService", "MsbService",

	# core msb_auth imports
	"TokenUser", "SessionData", "api_permissions", "Permissions",

	# core msb_http imports
	"ApiResponse", "RequestInfo", "RequestHeaders", "RestResponse",

	# core msb_exceptions imports
	"ApiException", "CrudApiException", "AppException", "ApiServiceExceptions",

	# core msb_validation imports
	"DefaultRules", "ValidationSchema", "InputField", "Validate", "validation_schema_wrapper", "api_inputs",

	# core msb_db imports
	"MsbModel", "MsbModelMetaFields", "MsbModelManager", "model_fields",

	# core msb_dataclasses imports
	"SearchParameter", "SearchParameterRequest", "Singleton",

	# others
	"Cipher"
]



class NotificationData:

	def __init__(self, **kwargs):
		self.subject = kwargs.get('subject', "")
		self.template = kwargs.get('template', "")
		self.recipients = kwargs.get('recipients', [])
		self.context = kwargs.get('context', {})
		self.body = kwargs.get('body', "")
		self.attachments = kwargs.get('attachments', [])

	def set_body(self, body):
		self.body = body
		return self

	def set_subject(self, subject):
		self.subject = subject
		return self

	def set_template(self, template):
		self.template = template
		return self

	def add_recipients(self, *recipients):
		self.recipients.extend(recipients)
		return self

	def add_context(self, **context):
		self.context.update(context)
		return self

	def add_file(self, path: str, name: str = None, content_type: str = None):
		self.attachments.append(dict(path=path, filename=name, content_type=content_type))
		return self


class EmailNotificationData(NotificationData):

	def __init__(self, **kwargs):
		super(EmailNotificationData, self).__init__(**kwargs)
		self.sender = kwargs.get('sender', "")
		self.cc = kwargs.get('cc', [])
		self.bcc = kwargs.get('bcc', [])
		self.priority = kwargs.get('priority', 1)

	def set_sender(self, sender):
		self.sender = sender
		return self

	def add_cc(self, *cc):
		self.cc.extend(cc)
		return self

	def add_bcc(self, *bcc):
		self.bcc.extend(bcc)
		return self

	def add_attachment_list(self, **attachements):
		for attachement in attachements:
			self.add_file(**attachement)

	def set_priority(self, priority):
		self.priority = priority
		return self


class YammerNotificationData(NotificationData):

	def __init__(self, **kwargs):
		super(YammerNotificationData, self).__init__(**kwargs)
		self.body = kwargs.get('body', "")
		self.cc = kwargs.get('cc', [])



import datetime
from dataclasses import dataclass
from typing import (List, Dict)


@dataclass
class UserAccountType:
	id: int
	name: str


@dataclass
class UserRole:

	def __init__(self, **kwargs):
		self.id = kwargs.get('id')
		self.name = kwargs.get('name')
		self.access_type = kwargs.get('access_type')

	def set_role(self, id: int, name: str, access_type: int):
		self.id, self.name, self.access_type = (id, name, access_type)
		return self


@dataclass
class UserLocation:

	def __init__(self, **kwargs):
		self.l_key = kwargs.get('l_key')
		self.name = kwargs.get('name')
		self.c_key = kwargs.get('c_key')
		self.country = kwargs.get('country')

	def set_location(self, id: int, name: str):
		self.l_key, self.name = (id, name)
		return self

	def set_country(self, id: int, name: str):
		self.c_key, self.country = (id, name)
		return self


@dataclass
class MenuPermissions:

	def __init__(self, menu_id: str = None, permission_str: str = ''):
		self.menu_id = menu_id
		self.permission_str = permission_str if (isinstance(permission_str, str) and len(permission_str) > 0) else "0000"

	@property
	def can_create(self):
		return bool(self.permission_str[0])

	@property
	def can_read(self):
		return bool(self.permission_str[1])

	@property
	def can_update(self):
		return bool(self.permission_str[2])

	@property
	def can_delete(self):
		return bool(self.permission_str[3])


@dataclass
class Reporties:
	def __init__(self, *reporties):
		self._reporties: list = list(reporties)

	def add(self, *reporties):
		self._reporties.extend(list(reporties))


class SessionData:

	def to_json(self):
		return {k: (v.__dict__ if hasattr(v, '__dict__') else v) for k, v in self.__dict__.items()}

	def __parsed_token_data(self, value, data_type):
		if not isinstance(value, data_type):
			return data_type(**value) if isinstance(value, dict) else data_type(*value)
		return value

	def __init__(self, id: id = None, email: str = None, username: str = None, **kwargs):
		self.id: int = id
		self.email: str = email
		self.username: str = username

		self.work_id: str = kwargs.get('work_id', '')
		self.image: str = kwargs.get('image', '')
		self.has_management_access: bool = kwargs.get("has_management_access", False)
		self.permissions: Dict = dict()
		self.login_time = datetime.datetime.now().__str__()
		self.auth_type = kwargs.get('auth_type', None)

		self.location: UserLocation = self.__parsed_token_data(kwargs.get('location', {}), UserLocation)
		self.role: UserRole = self.__parsed_token_data(kwargs.get('role', {}), UserRole)

		self.subscriptions: List = kwargs.get('subscriptions', [])
		self.dr_list: List = kwargs.get('dr_list', [])
		self.is_authenticated = kwargs.get('is_authenticated', False)

	def add_permission(self, menu_id, permission):
		self.permissions[menu_id] = permission

	def add_subscription(self, *subscriptiions):
		self.subscriptions.extend(subscriptiions)

	def set_has_management_access(self, status: bool = False):
		if (not self.has_management_access) and (status):
			self.has_management_access = status

	def add_reporties(self, *reporties):
		# if len(reporties) > 0:
		self.dr_list.extend(list(reporties))

	def set_is_authenticated(self, status):
		self.is_authenticated = status

	def set_auth_type(self, auth_type: str):
		self.auth_type = auth_type

	def get(self, key: str = '', default=None):
		return getattr(self, key, default)



DEFAULT_DATE_FORMAT = "%Y-%m-%d"
DEFAULT_TIME_FORMAT = "%H:%M:%S"
TIMESTAMP_FORMAT = f"{DEFAULT_DATE_FORMAT} {DEFAULT_TIME_FORMAT}"

DMY_DATE_FORMAT = "%d %m %Y"
DEFAULT_DATE_FORMAT_SLASHED = '%Y/%m/%d'
TIMESTAMP_FORMAT_SLASHED = f"{DEFAULT_DATE_FORMAT_SLASHED} {DEFAULT_TIME_FORMAT}"
HUMAN_FORMAT = "%a %d %b %y, %I:%M:%S %p"
DEFAULT_TIMEZONE = 'Asia/Kolkata'



import os

from cryptography.fernet import Fernet


class SymmetricCipher():

	def __init__(self, keyname: str = "SECRET_KEY", key: str = None):
		self.key = os.environ.get('SECRET_KEY', default='')

	def encrypt(self, data=None) -> str:
		if self.key is not None:
			try:
				data = str(data).encode() if not isinstance(data, bytes) else data
				return Fernet(key=self.key).encrypt(data).decode()
			except Exception as e:
				pass

		return data

	def decrypt(self, data=None) -> str:
		if self.key is not None:
			try:
				if isinstance(data, bytes):
					data = str(data).encode()
				return Fernet(key=self.key).decrypt(data).decode()
			except Exception:
				pass
		return data



from .dataclasses import (DbVendorConfig)

DJANGO_MIGRATION_DB_VENDOR_CONFIG: dict[str, DbVendorConfig] = dict(
	sqlite=DbVendorConfig(
		query_to_list_tables="SELECT tbl_name FROM sqlite_master WHERE type='table'",
		excluded_tables=["sqlite_master", "sqlite_sequence"],
		query_to_drop_table="DROP TABLE IF EXISTS {table_name};",

	),

	postgresql=DbVendorConfig(
		query_to_list_tables="select table_name from information_schema.tables WHERE table_schema in ('public')",
		excluded_tables=[],
		query_to_drop_table="DROP TABLE IF EXISTS {table_name} CASCADE;",
	)

)

REGEX_TO_SELECT_ENV_VARIABLE = r"(\s)*?([\w_]*)=(['\"])?([\w\d()\-:/*?=@.+!%$_#^&,]*)(['\"])?([\s\r\n]*)"
REGEX_TO_REPLACE_ENV_VARIABLE = r'\1\2=\3\2_VALUE\5\6'



class AuthResult:
	auth_type: str = None
	userid: str
	username: str
	email: str

	def __getitem__(self, item):
		return self.auth_data.get(item)

	def __init__(self, auth_username: str, auth_userid: str | int, email=None, auth_type: str = None, **kwargs):
		self.userid = auth_userid
		self.username = auth_username
		self.email = email
		self.auth_type = auth_type

	@property
	def id(self):
		return self.userid

	@property
	def user(self):
		return self.id

	@property
	def is_authenticated(self) -> bool:
		return (self.username is not None and self.userid is not None)

	@property
	def auth_data(self):
		_auth_data = dict()
		_auth_data['auth_type'] = self.auth_type
		_auth_data['userid'] = self.id
		_auth_data['username'] = self.username
		_auth_data['is_authenticated'] = self.is_authenticated
		return _auth_data

	def set_auth_type(self, auth_type: str):
		self.auth_type = auth_type
		return self

	def set_userid(self, userid: str):
		self.userid = userid
		return self

	def set_username(self, username: str):
		self.username = username
		return self

	def set_email(self, email: str):
		self.email = email
		return self



from .api_service import ApiService
from .exceptions import ApiServiceExceptions, CrudApiException
from .msb_service import MsbService

__all__ = [
	"ApiService", "ApiServiceExceptions", "CrudApiException", "MsbService"
]



from django.db import models
from msb.cipher import Cipher

from .constants import (COLUMN_NAME_DELETED)


class MsbModelManager(models.Manager):
	_default_filters = {}

	def _get_filtered_queryset(self, **filters):
		_query_set = super(MsbModelManager, self).get_queryset()
		query_filters = {}
		for filtername, filtervalue in filters.items():
			if hasattr(self.model, filtername):
				query_filters[filtername] = filtervalue
		return _query_set.filter(**query_filters)

	def retrieve(self, *args, **kwargs):
		return super(MsbModelManager, self).get(*args, **kwargs)

	def get(self, *args, **kwargs):
		pk = kwargs.get('pk')
		filters = {**kwargs}
		if pk is not None and isinstance(pk, str):
			filters['pk'] = int(Cipher.decrypt(pk))
		return self.retrieve(*args, **filters)

	@property
	def deleted(self):
		return self._get_filtered_queryset(**{COLUMN_NAME_DELETED: True})

	@property
	def from_all(self):
		return super(MsbModelManager, self).get_queryset()

	def get_queryset(self):
		return self._get_filtered_queryset(**{COLUMN_NAME_DELETED: False, **self._default_filters})




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['msb',
 'msb.apis',
 'msb.auth',
 'msb.cipher',
 'msb.dataclasses',
 'msb.datetime',
 'msb.db',
 'msb.devtools',
 'msb.env',
 'msb.exceptions',
 'msb.files',
 'msb.http',
 'msb.logging',
 'msb.services',
 'msb.testing',
 'msb.validation']

package_data = \
{'': ['*']}

modules = \
['hrin_msb']
install_requires = \
['celery>=5.2.7,<6.0.0',
 'cerberus>=1.3.4,<2.0.0',
 'cffi>=1.15.1,<2.0.0',
 'cryptography>=38.0.3,<39.0.0',
 'django-celery-results>=2.5.1,<3.0.0',
 'django-cors-headers>=4.0.0,<5.0.0',
 'django>=4.1.3,<5.0.0',
 'djangorestframework-simplejwt>=5.2.2,<6.0.0',
 'djangorestframework>=3.14.0,<4.0.0',
 'pandas>=1.5.1,<2.0.0',
 'pdf2docx>=0.5.6,<0.6.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.1,<3.0.0',
 'xhtml2pdf>=0.2.8,<0.3.0']

setup_kwargs = {
    'name': 'hrin-msb',
    'version': '0.2.71',
    'description': '',
    'long_description': '# hrin-msb\n\n## Pre-requisites for setup\n\n1. `pip install poetry`\n\n## How To Build\n\n1. `poetry build`\n2. `poetry config http-basic.pypi __token__ <access-token>`\n3. `poetry publish`\n\n# Change Log\n\n### Version 0.1.1\n\n***\n\n### Version 0.1.2\n\n***\n\n### Version 0.1.3\n\n***\n\n1. Default serializer added to ApiView\n2. fixed incorrect import in _validators.py\n3. fixed msb_database_router\n4. fixed Config.is_local_env() not working\n5. moved devscripts -> devtools\n6. File Utils Added to utils/files\n7. "app_label" removed from "TestConfig" & "ApiTest" Classes\n8. Fixed Bug : \'LoggingModelManager\' object has no attribute \'_queryset_class\'\n9. Fixed : Logging Model not showing any records\n10. Fixed : str method for base model, & removed current_timestamp method from base model\n\n***\n\n## Version 0.1.4\n\n1. Fixed : ModuleNotFoundError: No module named \'pdf2docx\'\n2. Renamed “FileGenerator“ => “FileFactory”,\n3. Add `create_` Prefix in FileFactory methods\n4. Renamed MsbMetaModel -> MsbModelMetaFields\n5. Added validation decorators, and fixed bulk validation issuses\n6. Modified Logging Configuration Files\n7. removed utils package\n8. moved msb_core.wrappers.exceptions -> msb_exceptions\n9. Fixed : Base ApiViews and Crud Routes\n10. Searchparameter class refactored, search method added in ApiService Class\n\n***\n\n## Version 0.1.41\n\n1. Fixed: Crud operations not working with encrypted id\'s\n2. Package dependencies updated\n3. Validator test cases refactored\n\n***\n\n## Version 0.1.5x\n\n### -- Version 0.1.51\n\n1. dbrouter print statement removed\n2. datetime constants renamed (added _FORMAT to all fof them)\n3. Fixed the default argument in msb_exception which was causing "DESC :" to log even if desc was none\n4. Api service create methdhod not working correctly\n5. file logging handler is not registered in local env by default, we need to pass `emulate_prod=True` to add it\n6. SearchParameter class imported in package.__init__.py\n7. Fixed : test-cases are breaking because of logs databae\n8. added base unittest class, and modified unit_test to inherit djangoTestCase instead of unittest\n9. Added Validation Schema not defined exceptions\n10. Fixed init_django_app error, int datatype instead of str while setting environement variable.\n11. Added use_django decorator to use over classes/functions\n12. Fixed : MsbMetaModel not working\n13. MsbModel meta fields accessor added\n14. Poetry dependencies updated\n15. DMY_DATE_FORMAT added\n16. versioning changed\n\n### -- Version 0.1.52\n\n1. Fixed : MsbMetaFields not working\n2. Fixed : logging model lot of exceptions are thrown if table applicationlogs is not found\n3. Fixed : logging model exceptions file not found\n4. Fixed : db_migration throwing error if no migration dire is found\n5. renamed use_djnago -> use_django, default value for settinfs_dir is added to "app"\n6. renamend _query in msb_model to db_query\n7. field_type, and label added to configuration model\n8. unique_together constraint added to ConfigurationModel\n9. class DjangoMigration creates migration folder if it doesn\'t exists\n10. Added automatic fixture loading\n11. Fixed : msb_model.__str__() was not able to read the primary key value\n12. comma removed from msbMetamodels\n13. Cipher now supports encryption/decryption of list items\n14. SearchParameter modified to supoprt autmatic filter assignment\n15. Refactor `msb_auth` : TokenUser,AuthResult,Constants added\n16. Jwt Token Validation is strict now, it allows only same owner\n\n### -- Version 0.1.521\n\n1. Fixed improper import exception\n\n### -- Version 0.1.522\n\n1. Added Login Required to base viewset\n2. Added Config Object class to msb_dataclasses\n3. Added msb_http to the package\n4. Added MsbDatabaseRouter in init.py\n5. Fixed model.delete() is not working\n\n### -- Version 0.1.611\n\n1. Modified django migration script\n2. Aded devtools to msb_ext\n3. removed `use_django` decorator & added `requires_django` decorator\n4. added default values for metafields\n5. added InputFiled.time in validation schema types\n\n### -- Version 0.2.0\n\n1. Fixed Cipher.decrypt() returning bytes instead of str\n2. Changed `SearchParameter` class implementation.\n3. default offset & limit fixed in `SearchParameter` class\n\n### -- Version 0.2.2\n\n1. default values removed from model metafields\n2. Fixed `ModuleNotFoundError: No module named \'_asymmetric_cipher\'`\n3. Fixed fixtures loaded in wrong sequence\n4. Feature `api_response.exception()` now sends back internal server error for undefined exceptions.\n5. Fixed Token validation error\n6. Added `DefaultJwtAuthSettings()` class, to unify settings across the services\n7. Added automatic fixture loading for test cases.\n\n### -- Version 0.2.3\n\n1. msb_testing refactored\n2. added new package `msb_const`\n3. Optimized imports throughout\n4. Refactored `msb_devtools`, removed `msb_ext.devtools`\n5. `msb_devtools._exceptions` file removed\n6. Added constant to `mab_auth._constants`\n\n### -- Version 0.2.4\n\n1. Refactor : (Optimized Imports,Sonarlint warnings)\n2. Refactor : Moved msb_apis.wrappers -> msb_http\n\n### -- Version 0.2.5\n\n1. Refactor : removed `msb_ext`, as it served no purpose\n2. Fixed : token validation failing due to AUD & ISS claim\n3. Added default fixture paths to `msb_const.paths`\n\n### -- Version 0.2.6\n\n1. renamed the property `db_query` to `rows`, to make it easier to understand.\n2. added a mentod to deteremine if the current environment is either dev or test env\n3. CrudActions.all is now the default parameter value for `.as_view()` method.\n4. Crud routes not working for single fields\n5. Fixed `self.seriallizer` not working with `dict()`\n6. Implemented `list` Api, to return predefined db columns\n7. search parameter added in crud, searchparameters class refatored\n\n### -- Version 0.2.7\n\n1. Fixed : `_list_field_names` not working with properties & relations\n2. Added `search_validation_rules` in DefaultRules\n\n### -- Version 0.2.8\n\n1. Fixed : list api breaking for foreign keys\n2. Fixed : Search parameter not working with fields\n3. Fixed: automatic encryption of db provate fields, now you need to put `DB_ENCRYPT_PRIVATE_FIELDS = True` to achieve that.\n\n### -- Version 0.2.9\n\n1. Fixed: automatic encryption of db primary key fields, now you need to put `DB_PK_IS_PRIVATE_FIELD = True` to achieve\n   that.\n\n### -- Version 0.2.10\n\n1. Fixed : incorrect crud routes for [list,search]\n2. Refactored : msb_http\n3. Implemented : `MsbIntraServiceRequest` class\n4. Implemented : `ApiRouter` class\n\n### -- Version 0.2.11\n\n1. Fixed /<str:id> route not working\n2. Authentication is failing due to jwt-token owner mismatch\n\n### -- Version 0.2.12\n\n1. Fixed: Payload validation is failing for DELETE and UPDATE Request\n\n### -- Version 0.2.13\n\n1. Updated : dependencies\n2. Fixed : crud list was breaking if no list fields are found/declared in model\n3. Refactored : Added _list fields in ConfigurationModel\n\n### -- Version 0.2.14\n\n1. Fixed : env_setup->db_migration requires user input to run\n\n### -- Version 0.2.15\n\n1. Fixed : Crud list api is breaking with None\n\n### -- Version 0.2.16\n\n1. Refactored : owner verification in Jwt token can now be controlled through settings.py file, using following variables"\n   `MSB_JWT_TRUSTED_OWNERS = []`(skips validation for the mentioned ip\'s) and `MSB_JWT_VERIFY_OWNER = False` (turns on/off\n   the owner verification)\n2. Implemented : `MsbIntraServiceRequest` class in msb_http\n3. Update : `ApiService` now has `resource_name` attribute in the class, this helps in better exception messages from\n   default crud operations.\n4. Refactor : `MsbIntraServiceRequest`\n5. Update : default crud api\'s now returns detailed error messages\n6. Update : All api\'s now returns detailed error messages for any exceptions.\n\n### -- Version 0.2.17\n\n1. Update : `ApiService` now automatically takes the resource name if not provided.\n2. Fixed : `MsbIntraServiceRequest` not working through api-gateway\n\n### -- Version 0.2.18\n\n1. Fixed : query parameters not working in `MsbIntraServiceRequest`\n2. Fixed : search parameter query not working in `MsbIntraServiceRequest`\n3. Fixed : limit and offset not working in list Api\n4. Refactor: _dataclasses\n5. Refactor : modified api_view error handler\n\n### -- Version 0.2.19\n\n1. removed Self parameter, due to compatibility issue\n2. Fixed : list/retrieve api bug\n\n### -- Version 0.2.2xx\n\n1. Fixed : UserRole.__init__() got an unexpected keyword argument \'access_type\'\n2. Added: recover method in model\n3. Refactor : moved all constants to `msb_constants`\n',
    'author': 'Prakash Mishra',
    'author_email': 'prakash.mishra@intimetec.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from django.utils.functional import cached_property

from .constants import RoleAccessConst, RoleConst
from .exceptions import MsbAuthExceptions
from .token import AuthToken


class TokenUser(AuthToken):

	@cached_property
	def username(self):
		return self.session.get('username', None)

	@property
	def is_authenticated(self):
		return self.session.get('is_authenticated', False) and self.is_valid

	@cached_property
	def is_admin(self):
		return (self.role.id == RoleConst.ADMIN_ROLE_ID) and self.has_management_access

	@property
	def email(self):
		return self.session.get('email', None)

	@property
	def has_management_access(self) -> bool:
		return self.session.has_management_access

	@property
	def work_id(self) -> str:
		return self.session.work_id

	@property
	def image(self) -> str:
		return self.session.image

	@property
	def permissions(self) -> dict:
		return self.session.permissions

	@property
	def login_time(self):
		return self.session.login_time

	@property
	def location(self):
		return self.session.location

	@property
	def role(self):
		return self.session.role

	@property
	def subscriptions(self):
		return self.session.subscriptions

	@property
	def dr_list(self):
		return self.session.dr_list

	def has_access_to(self, path: str) -> bool:
		return path in self.session.subscriptions

	def validate_employee_data_access(self, user_ids) -> bool:
		""" This function raises an exception if the user is not authorized to access the employee data."""
		if self.role.access_type in RoleAccessConst.SELF:
			raise MsbAuthExceptions.UnauthorizedAccessToUserData

		if self.role.access_type in RoleAccessConst.DIRECT_REPORTEES:
			for userid in user_ids:
				if userid not in self.dr_list:
					raise MsbAuthExceptions.UnauthorizedAccessToUserData

		# TODO : Add support for other access types.(RESTRICTED_EVERYONE)
		return True

	def has_role(self, role) -> bool:
		return int(role) == (self.role.id)

	def has_role_permission(self, *permissions):
		return any([permission().has_role_permission(self) for permission in permissions])
